/*
 *  DMD - CLOCK V0.5
 *  Par Christophe Girardot Aka Aganyte
 *  Novembre 2017 
 */

/* Include des librairies et fichiers annexes */
#include  <avr/pgmspace.h> // librairie de gestion de la ROM de l'arduino
#include  <SD.h>  // Librairie de gestion de la carte SD
#include  "SPI.h" // Librairie de gestion de la liaison SPI (DOT)
#include  "configuration.h" // Fichier de configuration 
#include  "Wire.h" // librairie pour la gestion de l'I2C (RTC)
#include  "DS3231.h" // librairie de gestion de la RTC
#include  "transition.h" //  Fichier de stockage des animations de transition
#include  "font.h" // Fichier de stockage des fonts
#include  "Image.h" //  Fichier de stockage des images
#include  "hook.h" // Fichier de stockage des animations du hook
#include  "Batman.h" // Fichier de stockage des animations du Batman
#include  "startrek.h" // Fichier de stockage des animations du Startrek
#include  "mix.h" // Fichier de stockage des animations qui ne sont pas en rapport avec un flipper particulier
#include  "Dot_12816.h"  // librairie de gestion du DOT
#include  "SDdot.h"  // librairie de la carte SD appliquée au DOT

// Boucle de Setup
void setup() {

  /* Activer la liaison série*/
  Serial.begin(9600);

  /* Activer la liaison I2C (RTC) */
  Wire.begin();

  /* Déclaration des entrées/sorties */
  pinMode(LATCH, OUTPUT);
  pinMode(E, OUTPUT);
  pinMode(A, OUTPUT);
  pinMode(B, OUTPUT);
  pinMode(C, OUTPUT);
  pinMode(D, OUTPUT);

  /* Activer l'entrée libre pour la fonction Random */
  randomSeed(analogRead(0));

  /* Mise à jour de l'heure (décommenter, remplir et téléverser. puis ré-commenter et re-téléverser) */
  // setDS3231time(byte seconds, byte minutes, byte hours, byte dayOfWeek, byte dayOfMonth, byte months, byte years)

  /* Tester la carte SD */
  Serial.print("Test de la SD card...");
  SPI.begin();
  if (!SD.begin(10)) {
    Serial.println("Pas de carte SD ou probleme de communication avec le lecteur");
    SPI.endTransaction(); // Libérer la ligne SPI pour le DOT
    return; // Attention, cette commande quitte le setup si il n'y a pas de SD présente
  }
  root = SD.open(SD_folder);
  Serial.println("carte SD prete");
  SD_ready = true;
  SPI.endTransaction(); // Libérer la ligne SPI pour le DOT
}

// Boucle principale
void loop() 
{ 
//  // Test de l'animation Buffer
//  unsigned char number = Fill_Animation_Buffer();
//  unsigned char line=0;
//  for(unsigned int count = 0; count<2*256; count++)
//  {
//    Serial.print(Animation_Ram[count],HEX);
//    Serial.print(" ");
//    line++;
//    if(line == 16)
//    {
//      Serial.println("");
//      line = 0;
//    }
//  }
  
  // Mire
    Image_2Bpp(0, 3); // image à l'adresse N°0 pendant 3 secondes (exemple d'utilisation d'une image en 4bpp)
//
//  // Introduction (exemple d'utilisation de l'affichage en 2 lignes de 16 caractères)
//  Ligne_In_RAM(1, "DMD-CLOCK  V:0.4"); // Ligne 1
//  Ligne_In_RAM(2, "   BY AGANYTE   "); // Ligne 2
//  Refresh_in_millis(1000);  // Rafraichir l'écran pendant une seconde (1000 millisecondes)
//  Clear_Ram_Buffer(); // Effacer la mémoire vidéo

  // Boucle principale
  for(;;)
  {
    // Lire toutes les animations de la carte SD (si présente)
    while( SD_ready == true & SD_I_have_seen_everything == false ) 
      SD_Video(Fill_Animation_Buffer()); // Charge et Joue une animation
    SD_I_have_seen_everything = false;
    
    // Scrolling de Bienvenue
    //Scrolling_1line("        WELCOME IN THE AGANYTE'S GAMEROOM !!        "); // Exemple d'utilisation d'un scrolling horizontal

    // lire les animations en vrac
    for(unsigned char count = 0; count <MixIndex[0]; count++)
    {
      SelectAnimation(count,MixIndex,Mix);
      clock_or_temp();
    }

    // lire les animations du Batman
    for(unsigned char count = 0; count <BatmanIndex[0]; count++)
    {
      SelectAnimation(count,BatmanIndex,Batman);
      clock_or_temp();
    }

    // lire les animations du Startrek
    for(unsigned char count = 0; count <StartrekIndex[0]; count++)
    {
      SelectAnimation(count,StartrekIndex,Startrek);
      clock_or_temp();
    }

    // lire les animations du Hook
    for(unsigned char count = 0; count <HookIndex[0]; count++)
    {
      SelectAnimation(count,HookIndex,Hook);
      clock_or_temp();
    }

    // Winners don't use drugs 
    Image_1Bpp(3,2); // image à l'adresse N°3 pendant 2 secondes (exemple d'utilisation d'une image en 1bpp)
  }
}
