/*
 * 
 * Librairie de gestion d'un DOT de 128 x 16 Pixels - V0.5
 * Par Christophe Girardot Aka Aganyte - Novembre 2017 
 * 
 */

/* sous programme de gestion des lignes */
void LINE(unsigned char number)
{
  digitalWrite(D,(number>>3) & 0x01);
  digitalWrite(C,(number>>2) & 0x01);
  digitalWrite(B,(number>>1) & 0x01);
  digitalWrite(A,number & 0x01);
}

/* Envoyer et afficher 128 bits sur le dot (une ligne) depuis la mémoire vidéo avec choix de la nuance */
void Send_128Bits(unsigned char ligne,unsigned char shade)
{
  SPI.beginTransaction(settings); // activer la liaison SPI
  if(shade == 2)
  {
    /* Envoie de 16 * 8 bits en série */
    for ( unsigned char Count_SPI2 = 16 ; Count_SPI2>0 ; Count_SPI2-- )
      SPI.transfer(~Shade2_Display_Ram[Count_SPI2 + ligne -1]); // Envoie de 8 Bits en série
  }
  if(shade == 1)
  {  
      /* Envoie de 16 * 8 bits en série */
    for ( unsigned char Count_SPI2 = 16 ; Count_SPI2>0 ; Count_SPI2-- )
      SPI.transfer(~Shade1_Display_Ram[Count_SPI2 + ligne -1]); // Envoie de 8 Bits en série
  }
  if(shade == 0)
  {   
      /* Envoie de 16 * 8 bits en série */
    for ( unsigned char Count_SPI2 = 16 ; Count_SPI2>0 ; Count_SPI2-- )
      SPI.transfer(~Display_Ram[Count_SPI2 + ligne -1]); // Envoie de 8 Bits en série
  }
  SPI.endTransaction(); // Libérer la liaison SPI pour la SD
}

/* Envoyer et afficher 16 * 128 Bits (une trame) depuis la mémoire vidéo (en monochrome) */
void Refresh()
{
  for ( unsigned char count = 0; count<16 ; count++)
  {
    digitalWrite(LATCH,LOW); // Préparer le front montant pour le Latch
    Send_128Bits(count*16,0); // Envoyer 128 bits sur le dot et attendre pour l'affichage
    digitalWrite(E,HIGH); // Désactiver la ligne
    LINE(count);
    digitalWrite(LATCH,HIGH); // Transferer la donnée sur les latchs
    digitalWrite(E,LOW); // Activer la ligne
    delayMicroseconds(Brightness); // laisser à la ligne le temps d'imprimer la rétine
  }
  digitalWrite(E,HIGH); // désactiver la dernière ligne
}

/* Envoyer et afficher 16 * 128 Bits (une trame) depuis la mémoire vidéo (gestion des nuances)  */
void Refresh_multishade()
{
  for ( unsigned char count = 0; count<16 ; count++)
  {
    digitalWrite(LATCH,LOW); // Préparer le front montant pour le Latch
    Send_128Bits(count*16,2); // Envoyer 128 bits sur le dot et attendre pour l'affichage
    digitalWrite(E,HIGH); // Désactiver la ligne
    LINE(count); // Sélectionner la ligne
    digitalWrite(LATCH,HIGH); // Transferer la donnée sur les latchs
    digitalWrite(E,LOW); // Activer la ligne
    delayMicroseconds(full_led_contrast); // Ajustage du temps d'allumage d'une led à fond en microsecond (attention : à régler avec une mire)

    digitalWrite(LATCH,LOW); // Préparer le front montant pour le Latch
    Send_128Bits(count*16,1); // Envoyer 128 bits sur le dot et attendre pour l'affichage
    digitalWrite(E,HIGH); // Désactiver la ligne
    digitalWrite(LATCH,HIGH); // Transferer la donnée sur les latchs
    digitalWrite(E,LOW); // Activer la ligne
    delayMicroseconds(dark_grey_led_contrast); // Ajustage du temps d'allumage d'une led à moitié en microsecond (attention : à régler avec une mire)

    digitalWrite(LATCH,LOW); // Préparer le front montant pour le Latch
    Send_128Bits(count*16,0); // Envoyer 128 bits sur le dot et attendre pour l'affichage
    digitalWrite(E,HIGH); // Désactiver la ligne
    digitalWrite(LATCH,HIGH); // Transferer la donnée sur les latchs
    digitalWrite(E,LOW); // Activer la ligne
    delayMicroseconds(light_grey_led_contrast); // Ajustage du temps d'allumage d'une led au mini en microsecond (attention : à régler avec une mire)
  }
  digitalWrite(E,HIGH); // désactiver la ligne
}

/* Rafraichir l'écran pendant X millisecondes */
void Refresh_in_millis(unsigned int Millis)
{
  long Time = millis();
  while(true)
  {
    Refresh();
    if(millis()-Time > Millis) break;
  }
}

/* charger la mémoire vidéo avec 16 * 128 Bits (une trame) depuis la rom (mode 1 bit par pixel, image bi-color) */
void Image_1Bpp(unsigned char image,unsigned char Xsecond)
{
  for ( unsigned int count = 0 ; count < 256 ; count++) 
    Display_Ram[count] = pgm_read_byte(Image+(count+(256*image)));
  Refresh_in_millis(Xsecond * 1000);
}

/* Effacer la mémoire vidéo */
void Clear_Ram_Buffer()
{
  for ( unsigned int count = 0; count < 256 ; count++) 
    Display_Ram[count] = 0x00;
}

/* Fonction écriture d'une ligne de 1x8 caractères en RAM video */
void Big_Line_In_RAM(const char *chaine)
{
  unsigned int FontPosData;
  unsigned char Caractere = 0, count = 8;
  while(count!=0)
  {
    FontPosData=((*chaine++)-32)*32;  // FontPosData détermine la position dans font.h du caractére à afficher.
    for(unsigned char Count=0 ; Count<16; Count++) // Font de 16 lignes de haut
    {
      for(unsigned char cT=0; cT<2; cT++) // Font de 2 octets de large
      {
        Display_Ram[cT+(16*Count)+(2*Caractere)] = Robot[(FontPosData+2*Count)+cT];
      }
    }
    Caractere++; // Passer au caractère suivant
    count--;
  }
}

/* Scrolling d'une ligne de 8 caractères */
void Scrolling_1line(const char *chaine)
{
  while(chaine[7] !='\0')
  {
    Big_Line_In_RAM(chaine++);
    Refresh_in_millis(Scrolling_speed_time);
  }
  Clear_Ram_Buffer();
}

/* Inversion de 8 bits pour le mode 2 lignes de 16 caractères (font inversée) */
unsigned char bit_reverse8(unsigned char n)
{
   n = (n << 4) | (n >> 4);
   n = ((n << 2) & 0xCC) | ((n >> 2) & 0x33);
   n = ((n << 1) & 0xAA) | ((n >> 1) & 0x55);
   return n;
}

/* Fonction écriture d'une ligne en RAM par superposition avec choix de la ligne (mode 2 lignes de 16 caractères) */
void Ligne_In_RAM(unsigned char line, const char *chaine)
{
  unsigned int FontPosData;
  unsigned char Count, Buffer, Caractere = 0;
  while(*chaine !='\0')
  {
    FontPosData=((*chaine++)-32)*8; // FontPosData détermine la position dans font.h du caractére à afficher.

    for(Count=0 ; Count<8; Count++) // Font de 8 lignes de haut
    {
      Buffer = bit_reverse8(Font88[FontPosData+Count]);
      if(line == 1) Display_Ram[(16*Count)+Caractere] = Display_Ram[(16*Count)+Caractere] | Buffer;
      else Display_Ram[128+(16*Count)+Caractere] = Display_Ram[128+(16*Count)+Caractere] | Buffer;
    }
    Caractere++; // Passer au caractère suivant
  }
}

/* Choisir et jouer une animation de transition */
void Transition()
{
  unsigned long Time = millis();
  unsigned char image = 0, Transition_number = random(0,TransitionIndex[0]), start_number = TransitionIndex[(Transition_number*2)+1], frame_number = TransitionIndex[(Transition_number*2)+2];
  while(true) 
  {
    for ( unsigned int count = 0 ; count < 256 ; count++ ) 
      Display_Ram[count] = Display_Ram[count] | pgm_read_byte(Transition_data+((256*start_number)+count+(256*image)));
    Refresh();
    if( millis() - Time > Transition_frame_time )
    {
      image++;
      Time = millis();
    }
    if(image >= frame_number)
      break;
  }
  Clear_Ram_Buffer();
}

// Affiche l'heure pendant X secondes
void clock_Xsecond(unsigned int Xsecond)
{
    unsigned char flag = 0;
    long second_counter = millis(), separator_counter = millis();
    while(true)
    {
      printDate(); // intérroger le DS3231
      char buffer[8];
      if(flag==0) sprintf(buffer, "  %02d:%02d ", hour, minute);
      else sprintf(buffer, "  %02d %02d ", hour, minute);
      Big_Line_In_RAM(buffer);
      Refresh();  // Rafraichir l'écran avec le buffer vidéo
      // Scheduler
      if ( millis() - separator_counter > separator_speed ) 
      {
        flag = ~flag;
        separator_counter = millis();
      }
      if ( millis() - second_counter > Xsecond*1000 ) 
        break;
    }
}

// Affiche la temperature pendant X secondes
void temp_Xsecond(unsigned char Temp_Second)
{
  int temp = 0;
  for(unsigned char count = 0; count<Temp_Second*10; count++)
  {
    temp = printtemp();
    unsigned char deg = temp / 10;
    char buffer[8];
    sprintf(buffer, "  %d.%dC  ", temp/10, temp-(10*deg));
    Big_Line_In_RAM(buffer);
    Refresh_in_millis(100);
  }
}

// alterne l'affichage de l'heure et de la temperature
void clock_or_temp()
{
  if (flag_temp_clock == false)
    temp_Xsecond(Temp_Second);
  else 
    clock_Xsecond(Clock_Second);
  flag_temp_clock = ~flag_temp_clock;
}

/* afficher une vidéo sur le dot */
void Video_DOT(unsigned char Scrolling, long int Start, unsigned int Time,unsigned char repeat, const unsigned char *Array)
{
  int image, Count;
  if ( transition_enable == true) Transition();
  if(Scrolling == false) // Animation mode
  {
    for(unsigned char Buf = 0; Buf<repeat; Buf++)
    {
      for(image=0 ; image<Time ; image+=2)
      {
        // Charger les pages de nuances en RAM
        for(Count = 0 ; Count<256 ; Count++) Shade2_Display_Ram[Count] = pgm_read_byte(Array + (Count+Start*256+(256*image))) & pgm_read_byte(Array + (Count+Start*256+(256*(image+1)))) ;
        for(Count = 0 ; Count<256 ; Count++) Shade1_Display_Ram[Count] = pgm_read_byte(Array + (Count+Start*256+(256*(image+1)))) ;
        for(Count = 0 ; Count<256 ; Count++) Display_Ram[Count] = pgm_read_byte(Array + (Count+Start*256+(256*image))) ;
        // Afficher
        for(unsigned char fps = 0; fps <slow_animation_speed; fps++) // permet de ralentir la séquence 
          Refresh_multishade();
      }
    }
  }
  else // Scrolling mode
  {
    for(image=0 ; image<Time ; image++)
    {
      // Charger les pages de nuances en RAM
      for(Count = 0 ; Count<256 ; Count++) Shade2_Display_Ram[Count] = pgm_read_byte(Array + (Count+Start*256+(16*(image+Time+16))) ) & pgm_read_byte(Array + (Count+Start*256+(16*image)) ) ;
      for(Count = 0 ; Count<256 ; Count++) Shade1_Display_Ram[Count] = pgm_read_byte(Array + (Count+Start*256+(16*(image+Time+16))) ) ;
      for(Count = 0 ; Count<256 ; Count++) Display_Ram[Count] = pgm_read_byte(Array + (Count+Start*256+(16*image)) ) ;
      // Afficher
      for(unsigned char lps = 0; lps <slow_scrolling_speed; lps++) // permet de ralentir la séquence
        Refresh_multishade();
    }
  }
  Clear_Ram_Buffer();
}

/* Fonction de selection d'une animation */
void SelectAnimation(unsigned char number,const unsigned int *Pointer, const unsigned char *Array)
{
  Video_DOT(Pointer[number*4+1],Pointer[number*4+2],Pointer[number*4+3],Pointer[number*4+4],Array);
}

/* Afficher une image sur le dot pendant X secondes (mode 2 bits par pixel, image quadri-color) */
void Image_2Bpp(unsigned char adress, unsigned char Xsecond)
{
  unsigned int Count = 0;
  long Start_time = millis();
  // Charger les pages de nuances en RAM
  for(Count = 0 ; Count<256 ; Count++) Shade2_Display_Ram[Count] = pgm_read_byte(Image + Count + (adress*256)) & pgm_read_byte(Image + Count+((adress*256)+256)) ;
  for(Count = 0 ; Count<256 ; Count++) Shade1_Display_Ram[Count] = pgm_read_byte(Image + Count+((adress*256)+256)) ;
  for(Count = 0 ; Count<256 ; Count++) Display_Ram[Count] = pgm_read_byte(Image + Count+(adress*256)) ;
  // Afficher
  while(true)
  {
    Refresh_multishade();
    if ( (millis() - Start_time) > Xsecond * 1000)
      break;
  }
  Clear_Ram_Buffer();
}
