/*
 * Librairie de Gestion de la carte SD appliquée au DOT 128x16
 * Par Aganyte - Fevrier 2017 
 * 
 */

/* Conversion d'une lettre ASCII (de 0 à F) en octets */
unsigned char Ascii_to_hex(unsigned char Value)
{
    if( Value > '9' ) Value -= 55;
    else Value -= 48;
    return(Value);
}

/* lire une trame dans la carte SD et la charger à l'adresse équivalente dans le buffer d'animation */
bool Read_SD_Trame(unsigned char Actual_frame)
{
  bool Finish = false;
  unsigned char MSB, LSB, ligne = 0;
  SPI.begin(); // Activer la liaison SPI
  for(unsigned int count = 0; count < 256; count++)
  {
    MSB = Ascii_to_hex(myFile.read()); // Lire et convertir le MSB
    LSB = Ascii_to_hex(myFile.read()); // Lire et convertir le LSB
    Animation_Ram[(Actual_frame*256)+count] = (MSB<<4) | LSB; // Charger le tout dans le buffer Animation
    ligne++;
    if (ligne == 16)
    {
      myFile.read(); myFile.read(); // Ignorer le retour chariot en fin de ligne
      ligne = 0;
    }
  }
  if( myFile.read() == '#') Finish = true; 
  myFile.read(); // Préparer la lecture de la trame suivante
  SPI.endTransaction(); // Libérer la liaison SPI pour le DOT
  return(Finish);
}

/* Gestion des fichiers de la carte SD */
void Open_SD_File()
{
  myFile = root.openNextFile(); // Ouvrir le prochain fichier sur la SD
  if (! myFile) // Si il n'y a pas de prochain fichier...
  {
    root.rewindDirectory(); // ...revenir au début du répertoire...
    if (read_just_SD == true) // ...Si on ne doit lire que les anims de la carte SD...
      myFile = root.openNextFile(); // ...Ouvrir le prochain fichier sur la SD...
    else // ...Sinon...
    {
      SD_I_have_seen_everything = true; // ...On a vu toutes les animations du répertoire
      return; // ... On quitte la fonction
    }
  }
  while (myFile.isDirectory()) // tant que le prochain fichier est un dossier...
    myFile = root.openNextFile(); // ...Ouvrir le prochain fichier sur la SD
  Serial.println(myFile.name()); // Afficher le nom du fichier ouvert dans la console
}

/* Charger l'animation buffer avec la carte SD tout en affichant l'heure */
unsigned char Fill_Animation_Buffer()
{
  bool separator_flag = false, Finish = false, Wait_time = false;
  unsigned char Actual_frame = 0;
  long separator_counter = millis(), Time = millis();
  Open_SD_File(); // Ouvrir le prochain fichier à lire
  if(SD_I_have_seen_everything == true) return(Actual_frame); // Si plus de fichier à lire, on quitte
  Clear_Ram_Buffer(); // Effacer l'écran
  printDate(); // Interroger le DS3231
  /* Telecharger la séquence */
  while(1)
  {  
    if( Finish == false ) // Si la séquence n'est pas totalement chargé...
      Finish = Read_SD_Trame(Actual_frame++); // ...charger une image de la séquence (mesuré à 8 ms max)
    else
      printDate(); // Interroger le DS3231   
    char buffer[8];
    if(separator_flag==0) sprintf(buffer, "  %02d:%02d ", hour, minute);
    else sprintf(buffer, "  %02d %02d ", hour, minute);
    Big_Line_In_RAM(buffer);    
    Refresh();  // Rafraichir l'écran avec le buffer vidéo
    
    /* Scheduler */
    if ( millis() - separator_counter > separator_speed ) // "separator_flag = ~separator_flag" ne marche pas ici...ja pas compris ?!
    {
      if(separator_flag == false)
        separator_flag = true;
      else
        separator_flag = false;
      separator_counter = millis();
    }
    if ( millis() - Time > Clock_Second * 1000 ) 
      Wait_time = true;
    if( Finish == true & Wait_time == true )
      break;
  }
  Transition(); // Lancer la transition avant la séquence
  return(Actual_frame);
}

/* afficher une vidéo sur le dot depuis le buffer de la carte SD */
void SD_Video(unsigned char Time)
{
  int image, Count;
  for(image=0 ; image<Time ; image+=2)
  {
    for(unsigned char fps = 0; fps <slow_animation_speed; fps++) // permet d'accélerer ou ralentir la séquence
    {
      for(unsigned char contrast = 0; contrast < full_led_contrast; contrast ++)
      {
        for(Count = 0 ; Count<256 ; Count++) Display_Ram[Count] = Animation_Ram[Count+(256*image)] & Animation_Ram[Count+(256* (image+1) )] ;
        Refresh();
      }
      for(unsigned char contrast = 0; contrast < dark_grey_led_contrast; contrast ++)
      {
        for(Count = 0 ; Count<256 ; Count++) Display_Ram[Count] = Animation_Ram[Count+(256* (image+1) )] ;
        Refresh();
      }
      for(Count = 0 ; Count<256 ; Count++) Display_Ram[Count] = Animation_Ram[Count+(256*image)] ;
      Refresh();
    }
  }
}
